global coffea_udf

@fn.pandas_udf(BinaryType(), fn.PandasUDFType.SCALAR)
def coffea_udf({% for col in cols %}{{col}}{{ "," if not loop.last }}{% endfor %}):
    global processor_instance, lz4_clevel
    
    columns = [{% for col in cols %}{{col}}{{ "," if not loop.last }}{% endfor %}]
    names = [{% for col in cols %}{{"'"|safe+col+"'"|safe}}{{ "," if not loop.last }}{% endfor %}]
    
    for i,col in enumerate(columns):
        #numpy array
        if names[i] == 'dataset' or columns[i].array[0].base is None:
            columns[i] = columns[i].values
        else:
            columns[i] = columns[i].array[0].base

    size = dataset.values.size
    items = {name:col for name,col in zip(names,columns)}
    
    df = processor.PreloadedDataFrame(size=size, items=items)
    df['dataset'] = dataset[0]
    
    vals = processor_instance.process(df)
    
    valsblob = lz4f.compress(cpkl.dumps(vals),compression_level=lz4_clevel)
    
    outs = np.full(shape=(size,),fill_value=b'',dtype='O')
    outs[0] = valsblob
    
    return pd.Series(outs)



class ParslBaseExecutor(object):

    def __init__(self, config, items, function, accumulator, workers, status, unit, desc):
        # create dfk

        # prepare the processor

        # submit jobs

        # collect them

        # reset?

        # kill the dfk?

        raise NotImplementedError



def parsl_executor(config, items, function, accumulator, workers, status, unit, desc):
    # create dataflow kernel

    # submit jobs

    # collect them

    raise NotImplementedError
    return accumulator


default_condor_cfg = None


def condor_executor(items, function, accumulator, workers, status=True, unit='items', desc='Processing', cfg=default_condor_cfg):
    return parsl_executor(cfg, items, function, accumulator, workers, status, unit, desc)


default_slurm_cfg = None


def slurm_executor(items, function, accumulator, workers, status=True, unit='items', desc='Processing', cfg=default_slurm_cfg):
    return parsl_executor(cfg, items, function, accumulator, workers, status, unit, desc)
